-----------------------------------------------------------------------------
-- INTEL DEVELOPER'S SOFTWARE LICENSE AGREEMENT
--
-- BY USING THIS SOFTWARE, YOU ARE AGREEING TO BE BOUND
-- BY THE TERMS OF THIS AGREEMENT.  DO NOT USE THE SOFTWARE
-- UNTIL YOU HAVE CAREFULLY READ AND AGREED TO THE FOLLOWING
-- TERMS AND CONDITIONS.  IF YOU DO NOT AGREE TO THE TERMS
-- OF THIS AGREEMENT, PROMPTLY RETURN THE SOFTWARE PACKAGE
-- AND ANY ACCOMPANYING ITEMS.
--
-- IF YOU USE THIS SOFTWARE, YOU WILL BE BOUND BY THE TERMS
-- OF THIS AGREEMENT.
--
-- LICENSE:  Intel Corporation ("Intel") grants you
-- the non-exclusive right to use the enclosed software
-- program ("Software").  You will not use, copy, modify,
-- display, rent, sell or transfer the Software or any portion
-- thereof, except as provided in this Agreement.
--
-- System OEM Developers may:
-- 1.      copy the Software for internal support, backup
--         or archival purposes;
-- 2.      internally install, use, display, or distribute
--         Intel owned Software in object code format;
-- 3.      internally modify Software source code that
--         Intel makes available to you for internal use
--         only as an OEM Developer;
-- 4.      internally install, use, display, modify, distribute,
--         and/or make derivatives ("Derivatives") of Intel owned
--         Software ONLY if you are a System OEM Developer and
--         NOT an end-user.
--
-- RESTRICTIONS:
--
-- YOU WILL NOT:
-- 1.      copy the Software, in whole or in part, except as
--         provided for in this Agreement;
-- 2.      decompile or reverse engineer any Software provided
--         in object code format;
-- 3.      distribute any Software or Derivative code to any
--         end-users, unless approved by Intel in a prior writing.
--
-- TRANSFER: You may not transfer the Software to any third
-- party without Intels prior written consent.
--
-- OWNERSHIP AND COPYRIGHT OF SOFTWARE: Title to the Software
-- and all copies thereof remain with Intel or its vendors.
-- The Software is copyrighted and is protected by United States
-- and international copyright laws.  You will not remove the
-- copyright notice from the Software.  You agree to prevent
-- any unauthorized copying of the Software.
--
-- DERIVATIVE WORK: OEM Developers that make Derivatives will
-- not be required to provide Intel with a copy of the source
-- or object code.  Any modification of Software shall be at
-- your sole risk and expense. No Software or Derivative
-- distribution to any third party is permitted under this
-- Agreement.
--
-- DUAL MEDIA SOFTWARE: If the Software package contains
-- multiple media, you may only use the medium appropriate
-- for your system.
--
-- WARRANTY: Intel warrants that it has the right to license
-- you to use, modify, display, or distribute the Software as
-- provided in this Agreement. The Software is provided "AS IS"
-- without WARRANTY of any kind.  Intel makes no representations
-- to upgrade, maintain, or support the Software at any time.
--
--
-- THE ABOVE WARRANTIES ARE THE ONLY WARRANTIES OF ANY
-- KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WARRANTIES
-- OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE
-- OR INFRINGEMENT OF ANY PATENT, COPYRIGHT OR OTHER
-- INTELLECTUAL PROPERTY RIGHT.
--
-- LIMITATION OF LIABILITY: NEITHER INTEL NOR ITS
-- VENDORS OR AGENTS SHALL BE LIABLE FOR ANY LOSS
-- OF PROFITS, LOSS OF USE, LOSS OF DATA, INTERRUPTION
-- OF BUSINESS, NOR FOR INDIRECT, SPECIAL, INCIDENTAL
-- OR CONSEQUENTIAL DAMAGES OF ANY KIND WHETHER UNDER
-- THIS AGREEMENT OR OTHERWISE, EVEN IF ADVISED OF THE
-- POSSIBILITY OF SUCH DAMAGES.
--
-- TERMINATION OF THIS LICENSE: Intel reserves the right
-- to conduct or have conducted audits to verify your
-- compliance with this Agreement.  Intel may terminate
-- this Agreement at any time if you are in breach of
-- any of its terms and conditions.  Upon termination,
-- you will immediately destroy, and certify in writing
-- the destruction of, the Software or return all copies
-- of the Software and documentation to Intel.
--
-- U.S. GOVERNMENT RESTRICTED RIGHTS: The Software and
-- documentation were developed at private expense and
-- are provided with "RESTRICTED RIGHTS".  Use, duplication
-- or disclosure by the Government is subject to restrictions
-- as set forth in FAR52.227-14 and DFAR252.227-7013 et seq.
-- or its successor.
--
-- EXPORT LAWS: You agree that the distribution and
-- export/re-export of the Software is in compliance
-- with the laws, regulations, orders or other restrictions
-- of the U.S. Export Administration Regulations.
--
-- APPLICABLE LAW: This Agreement is governed by the
-- laws of the State of California and the United States,
-- including patent and copyright laws.  Any claim
-- arising out of this Agreement will be brought in
-- Santa Clara County, California.
--
-- Copyright 1996, Intel Corporation, All Rights Reserved
--
--   File name :  i28F016SC_bfm_bdy.vhd
--   Title     :  Intel i28F016SC Bus-Functional Model
--   Module    :  Package Body
--
--   Purpose   :  the body of the procedures for the BFM
--
--   Roadmap   :
-------------------------------------------------------------------------------
--   Modification History :
--          Date              Author    Revision  Comments
-- Wed Apr  5 11:06:39 1995   Rupinder Hundal     Rev A          Creation
--   copied over the i28f008s_bfm_bdy.vhd
-------------------------------------------------------------------------------

  
PACKAGE BODY bfm_pkg IS
  
-------------------------------------------------------------------------------
--  Procedure read_byte: A search is made through the linked list to find the
--  link that contains the data for the given address. If found, then that
--  data is sent out. If not then it is assumed that the location is erased
--  with data = "11111111".
-------------------------------------------------------------------------------
  PROCEDURE read_byte (VARIABLE      blk_addr    : IN integer;
				 VARIABLE        link_addr   : IN bit_vector(15 DOWNTO 0);
				 VARIABLE        link_data   : OUT bit_vector(7 DOWNTO 0);
				 VARIABLE      memory_block  : IN mem_blks)
  IS
  VARIABLE      temp_mem_ptr     : mem_element_ptr := memory_block(blk_addr);
  BEGIN

   IF (memory_block(blk_addr) = NULL) THEN
	 link_data := "11111111";

   ELSE

    read_loop : WHILE (temp_mem_ptr /= NULL) LOOP 
			    IF (temp_mem_ptr.addr = link_addr) THEN
				 link_data := temp_mem_ptr.data;
			EXIT read_loop;

		   ELSIF (temp_mem_ptr.nxt /= NULL) THEN
			temp_mem_ptr := temp_mem_ptr.nxt;

		   ELSE 
			link_data := "11111111";
			EXIT read_loop;

			    END IF;

    END LOOP read_loop;

   END IF;
  END read_byte;
  
-------------------------------------------------------------------------------
--  Procedure program_byte: If the data being programmed is "11111111" then
--  this procedure does nothing. Else,it reads the array to check if that
--  location has previously been written to. If the existing data read back
--  is "11111111" then a new link is created with the current data provided.
--  Else,an AND operation between the existing data and the current data is
--  done and the value stored in the location.
-------------------------------------------------------------------------------
  PROCEDURE program_byte (VARIABLE      blk_addr  : IN integer;
					 VARIABLE    link_addr   : IN bit_vector(15 DOWNTO 0);
					 VARIABLE    link_data   : IN bit_vector(7 DOWNTO 0);
					 VARIABLE   memory_block : INOUT mem_blks)
  IS
  VARIABLE      temp_mem_ptr     : mem_element_ptr := memory_block(blk_addr);
  VARIABLE      exist_data       : bit_vector(7 DOWNTO 0);
  BEGIN
    IF (link_data /= "11111111") THEN
	  read_byte(blk_addr,link_addr,exist_data,memory_block);
	  IF (exist_data = "11111111") THEN
		 temp_mem_ptr := NEW mem_element_rec;
		 temp_mem_ptr.addr := link_addr;
		 temp_mem_ptr.data := link_data;
		 temp_mem_ptr.nxt := memory_block(blk_addr);
		 memory_block(blk_addr) := temp_mem_ptr;

	  ELSE
	    program_loop : WHILE (temp_mem_ptr /= NULL) LOOP
				  IF (temp_mem_ptr.addr = link_addr) THEN
					 temp_mem_ptr.data := temp_mem_ptr.data AND link_data;
			 EXIT program_loop;
				  END IF;
				  temp_mem_ptr := temp_mem_ptr.nxt;
	    END LOOP program_loop;
	  END IF;
    END IF;

  END program_byte;
  
-------------------------------------------------------------------------------
--  Procedure erase_block:The entire linked list associated with the given
--  block address is deallocated one at a time starting from the head of the
--  list.
-------------------------------------------------------------------------------
  PROCEDURE erase_block ( VARIABLE      blk_addr      : IN integer;
					 VARIABLE      memory_block  : INOUT mem_blks)
  IS
  VARIABLE      temp_mem_ptr     : mem_element_ptr := memory_block(blk_addr);
  BEGIN
    erase_loop : WHILE (memory_block(blk_addr) /= NULL) LOOP
			    temp_mem_ptr := memory_block(blk_addr);
			    memory_block(blk_addr) := temp_mem_ptr.nxt;
			    DEALLOCATE(temp_mem_ptr);
    END LOOP erase_loop;
  END erase_block;
  
-------------------------------------------------------------------------------
--  Procedure check_addr:The address is checked to see if it is invalid. The
--  boolean addr_valid returns the result. If each of the address bits is
--  anything but a 0 or 1 then addr_valid is set to FALSE.
-------------------------------------------------------------------------------
  PROCEDURE check_addr (VARIABLE  mem_addr   : IN std_ulogic_vector(19 DOWNTO 0);
				    VARIABLE  addr_valid : OUT BOOLEAN)
  IS
  BEGIN
	check_loop : FOR i IN mem_addr'LEFT DOWNTO mem_addr'RIGHT LOOP
	
		  IF (mem_addr(i) /= '0' AND mem_addr(i) /= '1') THEN
			addr_valid := FALSE;
			RETURN;
		  END IF;
		  
	END LOOP check_loop;
	
	addr_valid := TRUE;
	RETURN;
	
  END check_addr;

-------------------------------------------------------------------------------
--  Procedure check_data: The data is checked to see if it is valid. If it is
--  anything but a 0 or 1 then the boolean data_valid is set to FALSE.
-------------------------------------------------------------------------------
  PROCEDURE check_data (VARIABLE  mem_data   : IN std_ulogic_vector(7 DOWNTO 0);
				    VARIABLE  data_valid : OUT BOOLEAN)
  IS
  BEGIN
	check_loop : FOR i IN mem_data'LEFT DOWNTO mem_data'RIGHT LOOP
	
		  IF (mem_data(i) /= '0' AND mem_data(i) /= '1') THEN
			data_valid := FALSE;
			RETURN;
		  END IF;
		  
	END LOOP check_loop;
	
	data_valid := TRUE;
	RETURN;
  END check_data;

-------------------------------------------------------------------------------
--  Function Int_to_BitVector: Convert an integer to bit_vector.
--   
-------------------------------------------------------------------------------
  CONSTANT        MaxIntBitLength          : integer := 32;
  FUNCTION Int_to_BitVector (CONSTANT   int_data         : IN integer;
					    CONSTANT   size             : IN natural)
					    RETURN bit_vector
  IS
  VARIABLE      bit_value : bit_vector(size - 1 DOWNTO 0);
  VARIABLE      temp      : integer;
  VARIABLE      temp_data : integer := int_data;
  VARIABLE      neg_flag  : BOOLEAN := FALSE;
  VARIABLE      rems       : bit_vector((MaxIntBitLength - 1) DOWNTO 0):=(OTHERS=>'0');
  BEGIN
    IF (size = 0) THEN
	 assert FALSE
	   report "Int_to_BitVector: vector size specified as zero."
	   severity error;
    END IF;

    IF (temp_data < 0) THEN
	 neg_flag := TRUE;
	 assert (not(neg_flag))
	   report "Int_to_BitVector: negative integer."
	   severity error;
	 temp_data := - temp_data;
    END IF;

    FOR i IN 0 TO (MaxIntBitLength - 1) LOOP
	 EXIT WHEN temp_data <= 0;
	 temp := temp_data/2;
	 rems(i) := bit'VAL(temp_data - (temp*2));
	 temp_data := temp;
    END LOOP;
    
    assert (temp_data = 0)
	 report "Int_to_BitVector: integer too large."
	 severity error;

    bit_value := rems(size - 1 DOWNTO 0);
    RETURN (bit_value);

  END Int_to_BitVector;

-------------------------------------------------------------------------------
--  Function BitVector_to_Int: Convert a bit_vector to integer.
--   
-------------------------------------------------------------------------------
  FUNCTION BitVector_to_Int (CONSTANT bv_data : IN bit_vector)
					   RETURN integer
  IS
  VARIABLE    size      : integer := bv_data'LENGTH;
  VARIABLE    data_shift: bit_vector(bv_data'LENGTH - 1 DOWNTO 0) := bv_data;
  VARIABLE    int_value : integer := 0;
  VARIABLE    success   : BOOLEAN := false;
  BEGIN
    IF (size = 0) THEN
	 assert false
	   report "BitVector_to_Int: input data vector has size zero."
	   severity error;
    END IF;

    convert_loop : LOOP 
    IF (size >= MaxIntBitLength) THEN
	 assert false
	   report "BitVector_to_Int: bitvector is greater than 31 bits."
	   severity error;
	 EXIT convert_loop;
    END IF;

    int_value := bit'POS(data_shift(size - 1));

    FOR i IN (size - 2) DOWNTO 0 LOOP
	 int_value := (int_value * 2) + bit'POS(data_shift(i));
    END LOOP;

    success := TRUE;
    EXIT convert_loop;
    END LOOP convert_loop;
    
    IF (success) THEN
	 RETURN (int_value);
    ELSE
	 RETURN (0);
    END IF;

  END BitVector_to_Int;

-------------------------------------------------------------------------------
--  Function HexStr_to_BitVector: Convert a hex string to a bit_vector.
--   
-------------------------------------------------------------------------------
  FUNCTION HexStr_to_BitVector (CONSTANT hex_str : IN STRING)
						 RETURN bit_vector
  IS
  CONSTANT      bit_vec_len   : integer := 4 * hex_str'LENGTH;
  CONSTANT      hex_char_len  : integer := 4;
  CONSTANT      hex_str_cp    : STRING(1 TO hex_str'LENGTH) := hex_str;

  VARIABLE      invalid      : BOOLEAN := false;
  VARIABLE      c            : character;
  VARIABLE      i,x          : integer;
  VARIABLE      bv           : bit_vector(1 TO bit_vec_len);
  VARIABLE      bv_result    : bit_vector(bit_vec_len - 1 DOWNTO 0);

  CONSTANT      bin_zero     : bit_vector(1 to 4) := "0000";
  CONSTANT      bin_one      : bit_vector(1 to 4) := "0001";
  CONSTANT      bin_two      : bit_vector(1 to 4) := "0010";
  CONSTANT      bin_three    : bit_vector(1 to 4) := "0011";
  CONSTANT      bin_four     : bit_vector(1 to 4) := "0100";
  CONSTANT      bin_five     : bit_vector(1 to 4) := "0101";
  CONSTANT      bin_six      : bit_vector(1 to 4) := "0110";
  CONSTANT      bin_seven    : bit_vector(1 to 4) := "0111";
  CONSTANT      bin_eight    : bit_vector(1 to 4) := "1000";
  CONSTANT      bin_nine     : bit_vector(1 to 4) := "1001";
  CONSTANT      bin_ten      : bit_vector(1 to 4) := "1010";
  CONSTANT      bin_eleven   : bit_vector(1 to 4) := "1011";
  CONSTANT      bin_twelve   : bit_vector(1 to 4) := "1100";
  CONSTANT      bin_thirteen : bit_vector(1 to 4) := "1101";
  CONSTANT      bin_fourteen : bit_vector(1 to 4) := "1110";
  CONSTANT      bin_fifteen  : bit_vector(1 to 4) := "1111";

  BEGIN
    IF (hex_str'LENGTH = 0) THEN
	 assert false
	  report "HexStr_to_BitVector: input hex string has zero length."
	  severity NOTE;
	 RETURN "";
    END IF;

    i := 0;
    FOR x IN 1 TO (hex_str'LENGTH) LOOP
	  c := hex_str_cp(x);
	  CASE c IS
	    WHEN '0'        => bv(i+1 TO i+ hex_char_len) := bin_zero;
	    WHEN '1'        => bv(i+1 TO i+ hex_char_len) := bin_one;
	    WHEN '2'        => bv(i+1 TO i+ hex_char_len) := bin_two;
	    WHEN '3'        => bv(i+1 TO i+ hex_char_len) := bin_three;
	    WHEN '4'        => bv(i+1 TO i+ hex_char_len) := bin_four;
	    WHEN '5'        => bv(i+1 TO i+ hex_char_len) := bin_five;
	    WHEN '6'        => bv(i+1 TO i+ hex_char_len) := bin_six;
	    WHEN '7'        => bv(i+1 TO i+ hex_char_len) := bin_seven;
	    WHEN '8'        => bv(i+1 TO i+ hex_char_len) := bin_eight;
	    WHEN '9'        => bv(i+1 TO i+ hex_char_len) := bin_nine;
	    WHEN 'A' | 'a'  => bv(i+1 TO i+ hex_char_len) := bin_ten;
	    WHEN 'B' | 'b'  => bv(i+1 TO i+ hex_char_len) := bin_eleven;
	    WHEN 'C' | 'c'  => bv(i+1 TO i+ hex_char_len) := bin_twelve;
	    WHEN 'D' | 'd'  => bv(i+1 TO i+ hex_char_len) := bin_thirteen;
	    WHEN 'E' | 'e'  => bv(i+1 TO i+ hex_char_len) := bin_fourteen;
	    WHEN 'F' | 'f'  => bv(i+1 TO i+ hex_char_len) := bin_fifteen;
	    WHEN NUL        => exit;
	    WHEN OTHERS     => 
					ASSERT (invalid)
					REPORT "HexStr_to_BitVector: Hex string contains an "
						 & "invalid character."
					SEVERITY ERROR;
					invalid := TRUE;
	  END CASE;
	  i := i + hex_char_len;
    END LOOP;

    IF (invalid) THEN
	 bv(1 to i) := (OTHERS =>'0');
    END IF;
    
    bv_result(i-1 DOWNTO 0) := bv(1 to i);
    RETURN (bv_result(i-1 DOWNTO 0));
    
  END HexStr_to_BitVector;
  
END bfm_pkg;
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
