-----------------------------------------------------------------------------
-- INTEL DEVELOPER'S SOFTWARE LICENSE AGREEMENT
--
-- BY USING THIS SOFTWARE, YOU ARE AGREEING TO BE BOUND
-- BY THE TERMS OF THIS AGREEMENT.  DO NOT USE THE SOFTWARE
-- UNTIL YOU HAVE CAREFULLY READ AND AGREED TO THE FOLLOWING
-- TERMS AND CONDITIONS.  IF YOU DO NOT AGREE TO THE TERMS
-- OF THIS AGREEMENT, PROMPTLY RETURN THE SOFTWARE PACKAGE
-- AND ANY ACCOMPANYING ITEMS.
--
-- IF YOU USE THIS SOFTWARE, YOU WILL BE BOUND BY THE TERMS
-- OF THIS AGREEMENT.
--
-- LICENSE:  Intel Corporation ("Intel") grants you
-- the non-exclusive right to use the enclosed software
-- program ("Software").  You will not use, copy, modify,
-- display, rent, sell or transfer the Software or any portion
-- thereof, except as provided in this Agreement.
--
-- System OEM Developers may:
-- 1.      copy the Software for internal support, backup
--         or archival purposes;
-- 2.      internally install, use, display, or distribute
--         Intel owned Software in object code format;
-- 3.      internally modify Software source code that
--         Intel makes available to you for internal use
--         only as an OEM Developer;
-- 4.      internally install, use, display, modify, distribute,
--         and/or make derivatives ("Derivatives") of Intel owned
--         Software ONLY if you are a System OEM Developer and
--         NOT an end-user.
--
-- RESTRICTIONS:
--
-- YOU WILL NOT:
-- 1.      copy the Software, in whole or in part, except as
--         provided for in this Agreement;
-- 2.      decompile or reverse engineer any Software provided
--         in object code format;
-- 3.      distribute any Software or Derivative code to any
--         end-users, unless approved by Intel in a prior writing.
--
-- TRANSFER: You may not transfer the Software to any third
-- party without Intels prior written consent.
--
-- OWNERSHIP AND COPYRIGHT OF SOFTWARE: Title to the Software
-- and all copies thereof remain with Intel or its vendors.
-- The Software is copyrighted and is protected by United States
-- and international copyright laws.  You will not remove the
-- copyright notice from the Software.  You agree to prevent
-- any unauthorized copying of the Software.
--
-- DERIVATIVE WORK: OEM Developers that make Derivatives will
-- not be required to provide Intel with a copy of the source
-- or object code.  Any modification of Software shall be at
-- your sole risk and expense. No Software or Derivative
-- distribution to any third party is permitted under this
-- Agreement.
--
-- DUAL MEDIA SOFTWARE: If the Software package contains
-- multiple media, you may only use the medium appropriate
-- for your system.
--
-- WARRANTY: Intel warrants that it has the right to license
-- you to use, modify, display, or distribute the Software as
-- provided in this Agreement. The Software is provided "AS IS"
-- without WARRANTY of any kind.  Intel makes no representations
-- to upgrade, maintain, or support the Software at any time.
--
--
-- THE ABOVE WARRANTIES ARE THE ONLY WARRANTIES OF ANY
-- KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WARRANTIES
-- OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE
-- OR INFRINGEMENT OF ANY PATENT, COPYRIGHT OR OTHER
-- INTELLECTUAL PROPERTY RIGHT.
--
-- LIMITATION OF LIABILITY: NEITHER INTEL NOR ITS
-- VENDORS OR AGENTS SHALL BE LIABLE FOR ANY LOSS
-- OF PROFITS, LOSS OF USE, LOSS OF DATA, INTERRUPTION
-- OF BUSINESS, NOR FOR INDIRECT, SPECIAL, INCIDENTAL
-- OR CONSEQUENTIAL DAMAGES OF ANY KIND WHETHER UNDER
-- THIS AGREEMENT OR OTHERWISE, EVEN IF ADVISED OF THE
-- POSSIBILITY OF SUCH DAMAGES.
--
-- TERMINATION OF THIS LICENSE: Intel reserves the right
-- to conduct or have conducted audits to verify your
-- compliance with this Agreement.  Intel may terminate
-- this Agreement at any time if you are in breach of
-- any of its terms and conditions.  Upon termination,
-- you will immediately destroy, and certify in writing
-- the destruction of, the Software or return all copies
-- of the Software and documentation to Intel.
--
-- U.S. GOVERNMENT RESTRICTED RIGHTS: The Software and
-- documentation were developed at private expense and
-- are provided with "RESTRICTED RIGHTS".  Use, duplication
-- or disclosure by the Government is subject to restrictions
-- as set forth in FAR52.227-14 and DFAR252.227-7013 et seq.
-- or its successor.
--
-- EXPORT LAWS: You agree that the distribution and
-- export/re-export of the Software is in compliance
-- with the laws, regulations, orders or other restrictions
-- of the U.S. Export Administration Regulations.
--
-- APPLICABLE LAW: This Agreement is governed by the
-- laws of the State of California and the United States,
-- including patent and copyright laws.  Any claim
-- arising out of this Agreement will be brought in
-- Santa Clara County, California.
--
-- Copyright 1996, Intel Corporation, All Rights Reserved
--
--   File name :  i28f016sc_bfm_ent.vhd
--   Title     :  []
--   Module    :  []
--
--   Purpose   :  []
--
--   Roadmap   :
-------------------------------------------------------------------------------
--   Modification History :
--          Date                Author          Revision        Comments
-- Wed Apr 03 10:30:59 1995     Rupinder Hundal Rev A           Creation
-------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;

LIBRARY std;
USE std.textio.ALL;

USE WORK.bfm_pkg.ALL;

ENTITY i28f016sc_bfm IS

  GENERIC(
	   verbose:             BOOLEAN := FALSE;               --  verbose mode
	   init_file:           STRING := "initfile.dat";       --  location of initialization file
	   mem_size:            integer := 20;                  --  16M memory
	   vccpad:              std_uV := 5 V;                  --  vcc=5V
	   vpppad:              std_uV := 12 V;                 --  vpp=12V

	   twhqv1:              time := twhqv1_12_5;            --  min program time
	   twhqv2:              time := twhqv2_12_5;            --  min erase time
	   twhqv3:              time := twhqv3_12_5;            --  min lock-bit set time
	   twhqv4:              time := twhqv4_12_5;            --  min block lock-bit reset time
	   twhrh1:              time := twhrh1_12_5;            --  typ byte write suspend latency
	   twhrh2:              time := twhrh2_12_5;            --  typ erase suspend latency

 
	   --  read operation constants 
	 --tavav:               time := tavav_5;
	   tavqv:               time := tavqv_5;
	   telqv:               time := telqv_5;
	   tphqv:               time := tphqv_5;
	   tglqv:               time := tglqv_5;
	   telqx:               time := telqx_5;
	   tehqz:               time := tehqz_5;
	   tglqx:               time := tglqx_5;
	   tghqz:               time := tghqz_5;
	   toh:                 time := toh_5;

	   --  web controlled write timings
	 --tavav:               time := tavav_5;
	   tphwl:               time := tphwl_5;
	   telwl:               time := telwl_5;
	   twlwh:               time := twlwh_5;
	   tphhwh:              time := tphhwh_5;
	   tavwh:               time := tavwh_5;
	   tdvwh:               time := tdvwh_5;
	   twhdx:               time := twhdx_5;
	   twhax:               time := twhax_5;
	   twheh:               time := twheh_5;
	   twhwl:               time := twhwl_5;
	   twhrl:               time := twhrl_5;

	   --  ceb controlled write timings
	   tphel:               time := tphel_5;
	   twlel:               time := twlel_5;
	   teleh:               time := teleh_5;
	   tphheh:              time := tphheh_5;
	   taveh:               time := taveh_5;
	   tdveh:               time := tdveh_5;
	   tehdx:               time := tehdx_5;
	   tehax:               time := tehax_5;
	   tehwh:               time := tehwh_5;
	   tehel:               time := tehel_5

	  );

  PORT( ADDR:                   IN      std_ulogic_vector(20 DOWNTO 0);
	CEB:                    IN      std_ulogic;
	WEB:                    IN      std_ulogic;
	OEB:                    IN      std_ulogic;
	RPB:                    IN      std_ulogic;
	RD_BYB:                 OUT     std_logic := '1';
	DATA:                   INOUT   std_logic_vector(7 DOWNTO 0)
      );

END i28f016sc_bfm;
