-----------------------------------------------------------------------------
-- INTEL DEVELOPER'S SOFTWARE LICENSE AGREEMENT
--
-- BY USING THIS SOFTWARE, YOU ARE AGREEING TO BE BOUND
-- BY THE TERMS OF THIS AGREEMENT.  DO NOT USE THE SOFTWARE
-- UNTIL YOU HAVE CAREFULLY READ AND AGREED TO THE FOLLOWING
-- TERMS AND CONDITIONS.  IF YOU DO NOT AGREE TO THE TERMS
-- OF THIS AGREEMENT, PROMPTLY RETURN THE SOFTWARE PACKAGE
-- AND ANY ACCOMPANYING ITEMS.
--
-- IF YOU USE THIS SOFTWARE, YOU WILL BE BOUND BY THE TERMS
-- OF THIS AGREEMENT.
--
-- LICENSE:  Intel Corporation ("Intel") grants you
-- the non-exclusive right to use the enclosed software
-- program ("Software").  You will not use, copy, modify,
-- display, rent, sell or transfer the Software or any portion
-- thereof, except as provided in this Agreement.
--
-- System OEM Developers may:
-- 1.      copy the Software for internal support, backup
--         or archival purposes;
-- 2.      internally install, use, display, or distribute
--         Intel owned Software in object code format;
-- 3.      internally modify Software source code that
--         Intel makes available to you for internal use
--         only as an OEM Developer;
-- 4.      internally install, use, display, modify, distribute,
--         and/or make derivatives ("Derivatives") of Intel owned
--         Software ONLY if you are a System OEM Developer and
--         NOT an end-user.
--
-- RESTRICTIONS:
--
-- YOU WILL NOT:
-- 1.      copy the Software, in whole or in part, except as
--         provided for in this Agreement;
-- 2.      decompile or reverse engineer any Software provided
--         in object code format;
-- 3.      distribute any Software or Derivative code to any
--         end-users, unless approved by Intel in a prior writing.
--
-- TRANSFER: You may not transfer the Software to any third
-- party without Intels prior written consent.
--
-- OWNERSHIP AND COPYRIGHT OF SOFTWARE: Title to the Software
-- and all copies thereof remain with Intel or its vendors.
-- The Software is copyrighted and is protected by United States
-- and international copyright laws.  You will not remove the
-- copyright notice from the Software.  You agree to prevent
-- any unauthorized copying of the Software.
--
-- DERIVATIVE WORK: OEM Developers that make Derivatives will
-- not be required to provide Intel with a copy of the source
-- or object code.  Any modification of Software shall be at
-- your sole risk and expense. No Software or Derivative
-- distribution to any third party is permitted under this
-- Agreement.
--
-- DUAL MEDIA SOFTWARE: If the Software package contains
-- multiple media, you may only use the medium appropriate
-- for your system.
--
-- WARRANTY: Intel warrants that it has the right to license
-- you to use, modify, display, or distribute the Software as
-- provided in this Agreement. The Software is provided "AS IS"
-- without WARRANTY of any kind.  Intel makes no representations
-- to upgrade, maintain, or support the Software at any time.
--
--
-- THE ABOVE WARRANTIES ARE THE ONLY WARRANTIES OF ANY
-- KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WARRANTIES
-- OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE
-- OR INFRINGEMENT OF ANY PATENT, COPYRIGHT OR OTHER
-- INTELLECTUAL PROPERTY RIGHT.
--
-- LIMITATION OF LIABILITY: NEITHER INTEL NOR ITS
-- VENDORS OR AGENTS SHALL BE LIABLE FOR ANY LOSS
-- OF PROFITS, LOSS OF USE, LOSS OF DATA, INTERRUPTION
-- OF BUSINESS, NOR FOR INDIRECT, SPECIAL, INCIDENTAL
-- OR CONSEQUENTIAL DAMAGES OF ANY KIND WHETHER UNDER
-- THIS AGREEMENT OR OTHERWISE, EVEN IF ADVISED OF THE
-- POSSIBILITY OF SUCH DAMAGES.
--
-- TERMINATION OF THIS LICENSE: Intel reserves the right
-- to conduct or have conducted audits to verify your
-- compliance with this Agreement.  Intel may terminate
-- this Agreement at any time if you are in breach of
-- any of its terms and conditions.  Upon termination,
-- you will immediately destroy, and certify in writing
-- the destruction of, the Software or return all copies
-- of the Software and documentation to Intel.
--
-- U.S. GOVERNMENT RESTRICTED RIGHTS: The Software and
-- documentation were developed at private expense and
-- are provided with "RESTRICTED RIGHTS".  Use, duplication
-- or disclosure by the Government is subject to restrictions
-- as set forth in FAR52.227-14 and DFAR252.227-7013 et seq.
-- or its successor.
--
-- EXPORT LAWS: You agree that the distribution and
-- export/re-export of the Software is in compliance
-- with the laws, regulations, orders or other restrictions
-- of the U.S. Export Administration Regulations.
--
-- APPLICABLE LAW: This Agreement is governed by the
-- laws of the State of California and the United States,
-- including patent and copyright laws.  Any claim
-- arising out of this Agreement will be brought in
-- Santa Clara County, California.
--
-- Copyright 1996, Intel Corporation, All Rights Reserved
--
--   File name :  i28F016SC_bfm_pkg.vhd
--   Title     :  [Intel i28f016sc Bus-Functional Model]
--   Module    :  [Package Declaration]
--
--   Purpose   :  [to declare the types and procedures for the BFM]
--
--   Roadmap   :
-------------------------------------------------------------------------------
--   Modification History :
--          Date                Author          Revision        Comments
-- Wed Apr  5 10:54:04 1995     Rupinder Hundal Rev2.0_0.0      Creation
--                    copied over the i28f008s_bfm_pkg.vhd
-- Wed Apr  5 11:00:00 1995     Rupinder Hundal Rev2.0_0.1      included csm types
--                    added std_uV Type
-------------------------------------------------------------------------------

LIBRARY ieee;
  use ieee.std_logic_1164.all;


PACKAGE bfm_pkg IS

TYPE mem_element_rec;
TYPE mem_element_ptr IS ACCESS mem_element_rec;
TYPE mem_element_rec IS
  RECORD
    addr: bit_vector(15 DOWNTO 0);
    data: bit_vector(7 DOWNTO 0);
    nxt: mem_element_ptr;
  END RECORD ;
  
TYPE mem_blks IS array(Natural Range <>) of mem_element_ptr;

TYPE init_file_rec IS 
RECORD
  addr: STRING(1 TO 6);
  data: INTEGER;
END RECORD;

---
  TYPE std_uV IS RANGE -20000000 TO 20000000            -- -20V to 20V
	UNITS
	  uv;                                           -- base unit is uV
	  mv = 1000 uv;
	  v  = 1000 mv;
	END UNITS;
---

PROCEDURE read_byte (VARIABLE   blk_addr     : IN integer;
		     VARIABLE   link_addr    : IN bit_vector(15 DOWNTO 0);
		     VARIABLE   link_data    : OUT bit_vector(7 DOWNTO 0);
		     VARIABLE   memory_block : IN mem_blks);

PROCEDURE program_byte (VARIABLE   blk_addr     : IN integer;
			VARIABLE   link_addr    : IN bit_vector(15 DOWNTO 0);
			VARIABLE   link_data    : IN bit_vector(7 DOWNTO 0);
			VARIABLE   memory_block : INOUT mem_blks);

PROCEDURE erase_block (VARIABLE   blk_addr     : IN integer;
		       VARIABLE   memory_block : INOUT mem_blks);

PROCEDURE check_addr (VARIABLE  mem_addr   : IN std_ulogic_vector(19 DOWNTO 0);
		      VARIABLE  addr_valid : OUT BOOLEAN);

PROCEDURE check_data (VARIABLE  mem_data   : IN std_ulogic_vector(7 DOWNTO 0);
		      VARIABLE  data_valid : OUT BOOLEAN);

FUNCTION Int_to_BitVector (CONSTANT    int_data  : IN integer;
			   CONSTANT    size      : IN natural)
			   RETURN bit_vector;

FUNCTION BitVector_to_Int (CONSTANT bv_data : IN bit_vector)
			   RETURN integer;

FUNCTION HexStr_to_BitVector (CONSTANT hex_str : IN STRING)
			     RETURN bit_vector;


  -----------------------------------------------------------------------------
  --  Declare constants for timing parameters.
  --  Use these to set the generic timing parameters in the BFM entity. 
  --  
  --  Key is datasheet name followed by _X where X is the supply voltage.
  --  
  --  Commented items are either listed in a separate table, or are
  --  duplicated in the datasheet, but are maintained so that this table 
  --  reads identically to the datasheet.
  -----------------------------------------------------------------------------

  -----------------------------------------------------------------------------
  --  3v timing constants taken from 28F016SC datasheet
  -----------------------------------------------------------------------------
  --  3v read operation constants
  CONSTANT      tavav_3          : time := 120 ns;--  min cycle time
  CONSTANT      Tavqv_3          : time := 120 ns;--  max tacc
  CONSTANT      telqv_3          : time := 120 ns;--  max tce
  CONSTANT      tphqv_3          : time := 600 ns;--  max wake up time to data
  CONSTANT      tglqv_3          : time :=  50 ns;--  max oeb to data valid
  CONSTANT      telqx_3          : time :=   0 ns;--  min ceb low to data driv
  CONSTANT      tehqz_3          : time :=  55 ns;--  max ceb high to data Z
  CONSTANT      tglqx_3          : time :=   0 ns;--  min oeb low to data driv
  CONSTANT      tghqz_3          : time :=  20 ns;--  max oeb high to data Z
  CONSTANT      toh_3            : time :=   0 ns;--  min output hold
  --  3v web controlled write timings
  CONSTANT      tphwl_3          : time :=   1 us;--  max wake up to web low
  CONSTANT      telwl_3          : time :=  10 ns;--  min ceb low to web low
  CONSTANT      twlwh_3          : time :=  50 ns;--  min web low pulse
  CONSTANT      tphhwh_3         : time := 100 ns;--  min rp# vhh setup to web high
  CONSTANT      tvpwh_3          : time := 100 ns;--  min vpp to web high
  CONSTANT      tavwh_3          : time :=  50 ns;--  min add to web high
  CONSTANT      tdvwh_3          : time :=  50 ns;--  min data to web high
  CONSTANT      twhdx_3          : time :=   5 ns;--  min data hold from web hi
  CONSTANT      twhax_3          : time :=   5 ns;--  min add hold from web hi
  CONSTANT      twheh_3          : time :=  10 ns;--  min ceb hold, web high
  CONSTANT      twhwl_3          : time :=  30 ns;--  min web high pulse
  CONSTANT      twhrl_3          : time := 100 ns;--  max web high to rdy low
  -- These constants are also VPP dependent; defined separately
  CONSTANT      twhgl_3          : time :=   0 ns;--  write rec. before read
  CONSTANT      tqvvl_3          : time :=   0 ns;--  vpp hold from rdy high
  CONSTANT      tqvph_3          : time :=   0 ns;--  rp# hold from rdy high
  --  3v ceb controlled write timings
  CONSTANT      tphel_3          : time :=   1 us;--  max wake up to ceb low
  CONSTANT      twlel_3          : time :=   0 ns;--  min web low to ceb low
  CONSTANT      teleh_3          : time :=  70 ns;--  min ceb low pulse
  CONSTANT      tphheh_3         : time := 100 ns;--  min rp# vhh setup to ceb high
  CONSTANT      tvpeh_3          : time := 100 ns;--  min vpp to ceb high
  CONSTANT      taveh_3          : time :=  50 ns;--  min add to ceb high
  CONSTANT      tdveh_3          : time :=  50 ns;--  min data to ceb high
  CONSTANT      tehdx_3          : time :=   5 ns;--  min data hold from ceb hi
  CONSTANT      tehax_3          : time :=   5 ns;--  min add hold from ceb hi
  CONSTANT      tehwh_3          : time :=   0 ns;--  min web hold, ceb high
  CONSTANT      tehel_3          : time :=  25 ns;--  min ceb high pulse
  CONSTANT      tehrl_3          : time := 100 ns;--  max ceb high to rdy low
  -- These constants are also VPP dependent; defined separately
  CONSTANT      tehgl_3          : time :=   0 ns;--  write rec. before read

  
  -----------------------------------------------------------------------------
  --  5v timing constants taken from 28F016SC datasheet;
  --  Fast part, 10% supply.
  -----------------------------------------------------------------------------
   --  5v read operation constants
  CONSTANT      tavav_5          : time := 100 ns;--  min cycle time
  CONSTANT      Tavqv_5          : time := 100 ns;--  max tacc
  CONSTANT      telqv_5          : time := 100 ns;--  max tce
  CONSTANT      tphqv_5          : time := 400 ns;--  max wake up time to data
  CONSTANT      tglqv_5          : time :=  45 ns;--  max oeb to data valid
  CONSTANT      telqx_5          : time :=   0 ns;--  min ceb low to data driv
  CONSTANT      tehqz_5          : time :=  55 ns;--  max ceb high to data Z
  CONSTANT      tglqx_5          : time :=   0 ns;--  min oeb low to data driv
  CONSTANT      tghqz_5          : time :=  10 ns;--  max oeb high to data Z
  CONSTANT      toh_5            : time :=   0 ns;--  min output hold
  --  5v web controlled write timings
  CONSTANT      tphwl_5          : time :=   1 us;--  max wake up to web low
  CONSTANT      telwl_5          : time :=  10 ns;--  min ceb low to web low
  CONSTANT      twlwh_5          : time :=  40 ns;--  min web low pulse
  CONSTANT      tphhwh_5         : time := 100 ns;--  min rp# vhh setup to web high
  CONSTANT      tvpwh_5          : time := 100 ns;--  min vpp to web high
  CONSTANT      tavwh_5          : time :=  40 ns;--  min add to web high
  CONSTANT      tdvwh_5          : time :=  40 ns;--  min data to web high
  CONSTANT      twhdx_5          : time :=   5 ns;--  min data hold from web hi
  CONSTANT      twhax_5          : time :=   5 ns;--  min add hold from web hi
  CONSTANT      twheh_5          : time :=  10 ns;--  min ceb hold, web high
  CONSTANT      twhwl_5          : time :=  30 ns;--  min web high pulse
  CONSTANT      twhrl_5          : time :=  90 ns;--  max web high to rdy low
  -- These constants are also VPP dependent; defined separately
  CONSTANT      twhgl_5          : time :=   0 ns;--  write rec. before read
  CONSTANT      tqvvl_5          : time :=   0 ns;--  vpp hold from rdy high
  CONSTANT      tqvph_5          : time :=   0 ns;--  rp# hold from rdy high
  --  5v ceb controlled write timings
  CONSTANT      tphel_5          : time :=   1 us;--  max wake up to ceb low
  CONSTANT      twlel_5          : time :=   0 ns;--  min web low to ceb low
  CONSTANT      teleh_5          : time :=  50 ns;--  min ceb low pulse
  CONSTANT      tphheh_5         : time := 100 ns;--  min rp# vhh setup to ceb high
  CONSTANT      tvpeh_5          : time := 100 ns;--  min vpp to ceb high
  CONSTANT      taveh_5          : time :=  40 ns;--  min add to ceb high
  CONSTANT      tdveh_5          : time :=  40 ns;--  min data to ceb high
  CONSTANT      tehdx_5          : time :=   5 ns;--  min data hold from ceb hi
  CONSTANT      tehax_5          : time :=   5 ns;--  min add hold from ceb hi
  CONSTANT      tehwh_5          : time :=   0 ns;--  min web hold, ceb high
  CONSTANT      tehel_5          : time :=  25 ns;--  min ceb high pulse
  CONSTANT      tehrl_5          : time :=  90 ns;--  max ceb high to rdy low
  -- These constants are also VPP dependent; defined separately
  CONSTANT      tehgl_5          : time :=   0 ns;--  write rec. before read


  -----------------------------------------------------------------------------
  --  2.7v timing constants taken from 28F016SC datasheet.
  -----------------------------------------------------------------------------
  --  2.7v read operation constants
  CONSTANT      tavav_27          : time := 150 ns;--  min cycle time
  CONSTANT      Tavqv_27          : time := 150 ns;--  max tacc
  CONSTANT      telqv_27          : time := 150 ns;--  max tce
  -- all other parameters are equal to 3V values.


  -----------------------------------------------------------------------------
  --  Block Erase, Byte Write and Lock-Bit Configuration Performance.
  --  Timing constants taken from 28F016SC datasheet.
  -----------------------------------------------------------------------------
  -- 12v VPP / 5V VCC web/ceb controlled writes.
  CONSTANT      twhqv1_12_5      : time := 4.8 us;--  min program time
  CONSTANT      twhqv2_12_5      : time := 300 ms;--  min erase time
  CONSTANT      twhqv3_12_5      : time := 7.8 us;--  min lock-bit set time
  CONSTANT      twhqv4_12_5      : time := 300 ms;--  min block lock-bit reset time
  CONSTANT      twhrh1_12_5      : time :=   4 us;--  typ byte write suspend latency
  CONSTANT      twhrh2_12_5      : time := 9.6 us;--  typ erase suspend latency

  -- 5v VPP / 5V VCC web/ceb controlled writes.
  CONSTANT      twhqv1_5_5       : time := 6.5 us;--  min program time
  CONSTANT      twhqv2_5_5       : time := 900 ms;--  min erase time
  CONSTANT      twhqv3_5_5       : time := 9.5 us;--  min lock-bit set time
  CONSTANT      twhqv4_5_5       : time := 900 ms;--  min block lock-bit reset time
  CONSTANT      twhrh1_5_5       : time :=   5 us;--  typ byte write suspend latency
  CONSTANT      twhrh2_5_5       : time := 9.6 us;--  typ erase suspend latency



  -- 12v VPP / 3V VCC web/ceb controlled writes
  CONSTANT      twhqv1_12_3      : time := 6.7 us;--  min program time
  CONSTANT      twhqv2_12_3      : time := 800 ms;--  min erase time
  CONSTANT      twhqv3_12_3      : time := 9.7 us;--  min lock-bit set time
  CONSTANT      twhqv4_12_3      : time := 800 ms;--  min block lock-bit reset time
  CONSTANT      twhrh1_12_3      : time :=   5 us;--  typ byte write suspend latency
  CONSTANT      twhrh2_12_3      : time := 9.6 us;--  typ erase suspend latency

  -- 5v VPP / 3V VCC web/ceb controlled writes
  CONSTANT      twhqv1_5_3       : time := 8.2 us; --  min program time
  CONSTANT      twhqv2_5_3       : time := 1000 ms;--  min erase time
  CONSTANT      twhqv3_5_3       : time := 11.2 us;-- min lock-bit set time
  CONSTANT      twhqv4_5_3       : time := 1000 ms;--  min block lock-bit reset time
  CONSTANT      twhrh1_5_3       : time :=   5 us; --  typ byte write suspend latency
  CONSTANT      twhrh2_5_3       : time := 9.6 us; --  typ erase suspend latency

  -- 3v VPP / 3V VCC web/ceb controlled writes.
  CONSTANT      twhqv1_3_3       : time :=  15 us; --  min program time
  CONSTANT      twhqv2_3_3       : time := 1500 ms;--  min erase time
  CONSTANT      twhqv3_3_3       : time :=  18 us; --  min lock-bit set time
  CONSTANT      twhqv4_3_3       : time := 1500 ms;--  min block lock-bit reset time
  CONSTANT      twhrh1_3_3       : time :=   6 us; --  typ byte write suspend latency
  CONSTANT      twhrh2_3_3       : time := 16.2 us;--  typ erase suspend latency


END bfm_pkg;
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
